#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Displays *flooding* of e.g. gtopo30 or srtm90 DEM raster data.  Plots
data with the help of gdal (http://pypi.python.org/pypi/GDAL) and
matplotlib-basemap.

Version 0.1.0,  2012-12-08
Version 0.1.1,  2014-03-30
Version 0.1.2,  2014-04-21
Version 0.1.3,  2015-03-27
Version 0.2.0,  2018-07-01

Copyright (c) Dietmar Thaler

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.

E-Mail: info@foehnwall.at

Web: http://www.foehnwall.at/alpflood.html

**Usage**:

* Make sure you have installed

  - *python 2.7* (should be 3.x proof)

  - *matplotlib 1.1.1* or higher

  - *basemap 1.02* or higher

  - *python-gdal 1.7.3* or higher

  - *mencoder* or another movie-maker program

  and all their dependencies.

* Download *topoflooding* and *configfile_template.py* in a separate
  directory

* Make sure you have a DEM-file in a format supported by gdal (compare
  supported formats at

* edit *configfile_template.py* concerning to your needs to
  *yourconfigfile.py*

* Call ``python2 topoflooding.py yourconfigfile.py``

* call *mencoder* or any other movie-maker tool to make a movie of the
  image files

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals  # maybe problems with module csv

__version__ = '0.2.0 2018-07-01'

from mpl_toolkits.basemap import Basemap, cm
from numpy import ma
from osgeo import gdal  # , ogr
import os
import sys

import matplotlib.pyplot as plt
import numpy as np
import os.path as op

# Include configuration the file that contains valid
# python variables and omits the extension .py
try:
        cgfn = sys.argv[1]
except:
	print("Error: No or invalid config-file!")
	sys.exit(-2)
if op.isfile(cgfn) or op.isfile(cgfn + ".py"):
	configfilename = op.splitext(cgfn)[0]
else:
	print("Error: '" + cgfn + "' does not exits!")
	sys.exit(-1)
print("Import configfile " + configfilename)
exec('from %s import *' % configfilename)
print(configfilename, " imported")

# Check if dempath and picdir exist
if not op.isfile(dempath):
	print("ERROR: DEM-File ")
	print(dempath)
	print("does not exist")
	print("Exit to check " + cgfn)
	sys.exit()
if not op.isdir(picdir):
	print("WARNING: Output-Directory ")
	print(picdir)
	yn = input("does not exist. Create it (y/n)? ")
	if yn.lower() == 'y':
		os.makedirs(picdir)
		print("Directory created: "); print(picdir)
	else:
		print("Exit")
		sys.exit()

endheight = endheight + 1  # for looping reasons

print(50 * '-')
print("Input-Data from config-file: ", cgfn)
print("Headline: ", headline)
print("Copyright: ", copyright)
print("Path with DEM-Data: ", dempath)
print("Output directory for images: ", picdir)
print("Lower latitude: ", lllat)
print("Upper latitude: ", urlat)
print("Western longitude: ", lllon)
print("Eastern longitude: ", urlon)
print("Parallels (lat.)  increment to be plotted: ", lat_increment)
print("Meridians (lon.)  increment to be plotted: ", lon_increment)
print("Start height: ", startheight)
print("End height:   ", endheight)
print("Height step:  ", stepheight)
print("Boundary resoltion: ", boundaryresolution)
print("Regridding gridsize: ", regridsize)
print(50 * '-')

# Report gdal-Errors by Exceptions!
gdal.UseExceptions()
gdal.AllRegister()  # necessary????

# open DEM-Data
gd = gdal.Open(dempath)

# Request DEM-characteristics
cols = gd.RasterXSize
rows = gd.RasterYSize
bands = gd.RasterCount

geotransform = gd.GetGeoTransform()
originX = geotransform[0]
originY = geotransform[3]
pixelWidth = geotransform[1]
pixelHeight = geotransform[5]

# Output DEM-characteristics
print("No.    columns    : ", cols)
print("No.    rows       : ", rows)
print("No. raster-bands  : ", bands)
print("     Origin X     : ", originX)
print("     Origin Y     : ", originY)
print("Pixel Width  (deg): ", pixelWidth)
print("Pixel Height (deg): ", pixelHeight)
print(50 * '-')

# Prepare the map
print("Read as Array")
array = gd.ReadAsArray()
print("As Array Read")
coords = geotransform  # aus historischen Gruenden
nlons = array.shape[1]; nlats = array.shape[0]
lons = coords[0] + pixelWidth * np.arange(nlons)
lats = coords[3] + pixelHeight * np.arange(nlats)[::-1]  # reverse lats

print("Set up Basemap Instance")
# setup basemap instance. ########################################
"""# transposed Mercator projection
m = Basemap(projection = 'tmerc',resolution = boundaryresolution,\
    area_thresh = 5.0,\
    llcrnrlat = lllat, urcrnrlat = urlat,\
    llcrnrlon = lllon, urcrnrlon = urlon,\
    lon_0 = (lllon+urlon)/2.0, lat_0 = (lllat+urlat)/2.0)"""

# Mercator projection with true latitude lat_ts
m = Basemap(projection='merc', resolution=boundaryresolution, \
    area_thresh=5.0, \
    llcrnrlat=lllat, urcrnrlat=urlat, \
    llcrnrlon=lllon, urcrnrlon=urlon, \
    lat_ts=(lllat + urlat) / 2.0)

##################################################################
print("Basemap Instance Set up")

# create masked array, reversing data in latitude direction (so that
# data is oriented in increasing latitude, as transform_scalar
# requires).
print("Reverse Array")
topoin = ma.masked_values(array[::-1, :], -999.)
print("Array reversed")

# transform DEM data to a 0.5 km native projection grid
print("Transform to grid, may take a while!")
print("In case of a program freeze due to memory problems")
print("increase the value of the config-file variable 'reggridsize'!")
nx = int((m.xmax - m.xmin) / regridsize) + 1; ny = int((m.ymax - m.ymin) / regridsize) + 1
tp1 = m.transform_scalar(topoin, lons, lats, nx, ny, masked=True)
print("To grid Transformed")

# Make ocean height 0
tp0 = np.where(tp1 > 9999, 0, tp1)

print(50 * '-')
print("Iterate through all height levels")
# Iterate through all levels
hrange = np.arange(startheight, endheight, stepheight)
for h in hrange:
    header = (headline + ' > %04d m') % h
    print(header)
    # dry (true) or flooded (false):
    heightlevel = np.greater(tp0, h)
    # plot DEM image on map.
    im = m.imshow(heightlevel, vmin=-1, vmax=2, cmap=plt.cm.jet)

    # draw meridians, parallels etc..
    parallels = np.arange(round(lllat), round(urlat) + lat_increment, lat_increment)
    m.drawparallels(parallels, labels=[1, 0, 0, 0], labelstyle="+/-")
    meridians = np.arange(round(lllon), round(urlon) + lon_increment, lon_increment)
    m.drawmeridians(meridians, labels=[0, 0, 0, 1], labelstyle="+/-")
    m.drawcoastlines(color='blue')
    m.drawcountries(color='red')
    m.drawrivers(color='blue')

    # m.colorbar(im)
    plt.title(header, y=1.025)
    # plt.title(header)
    # 0,0 lower left, 1,1, upper right
    axx = plt.gca()
    plt.text(0.025, 0.025, copyright, fontsize=9, transform=axx.transAxes)
    fname = op.join(picdir, "topo%04d.jpg" % (h))
    plt.savefig(fname, facecolor='w', format='jpg', dpi=300)
    plt.close()  # Never forget to close
    plt.clf()  # and clear the plot
    # remove arrays
    heightlevel = np.nan
    im = np.nan
    print("Output to file: ", fname)
print(50 * '-')
print("Finished!")
print(50 * '-')
